using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Kav
{
    public class SimpleDepthEffect : Effect, IHasWorldMatrix
    {
        EffectParameter worldParam;
        EffectParameter viewProjectionParam;

        Matrix world;
        Matrix view;
        Matrix projection;

        EffectDirtyFlags dirtyFlags = EffectDirtyFlags.All;

        public Matrix World
        {
            get { return world; }
            set
            {
                world = value;
                dirtyFlags |= EffectDirtyFlags.World;
            }
        }

        public Matrix View
        {
            get { return view; }
            set
            {
                view = value;
                dirtyFlags |= EffectDirtyFlags.ViewProj;
            }
        }

        public Matrix Projection
        {
            get { return projection; }
            set
            {
                projection = value;
                dirtyFlags |= EffectDirtyFlags.ViewProj;
            }
        }

        public SimpleDepthEffect(GraphicsDevice graphicsDevice) : base(graphicsDevice, Resources.SimpleDepthEffect)
        {
            CacheEffectParameters();
        }

        protected override void OnApply()
        {
            if ((dirtyFlags & EffectDirtyFlags.World) != 0)
            {
                worldParam.SetValue(world);

                dirtyFlags &= ~EffectDirtyFlags.World;
            }

            if ((dirtyFlags & EffectDirtyFlags.ViewProj) != 0)
            {
                Matrix.Multiply(ref view, ref projection, out Matrix viewProjection);
                viewProjectionParam.SetValue(viewProjection);

                dirtyFlags &= ~EffectDirtyFlags.ViewProj;
            }
        }

        private void CacheEffectParameters()
        {
            worldParam = Parameters["World"];
            viewProjectionParam = Parameters["ViewProjection"];
        }
    }
}
